-- Note: this Mixin is a little bit special, it adds both a function in the core and others in the button mixin

function FlexBar2:RegisterTextSub(Name, Triggers, Func) -- The core part of RegisterTextSub (pretty short)
	if(type(self.TextSubs) ~= "table") then
		self.TextSubs = {};
	end
	if(type(Triggers) == "string") then
		if(type(self.TextSubs[Triggers]) ~= "table") then
			self.TextSubs[Triggers] = {};
		end
		self.TextSubs[Triggers][Name] = Func;
	elseif(type(Triggers) == "table") then
		for _, Trigger in ipairs(Triggers) do
			if(type(self.TextSubs[Trigger]) ~= "table") then
				self.TextSubs[Trigger] = {};
			end
			self.TextSubs[Trigger][Name] = Func;
		end
	end
	if(type(self.TextFuncs) ~= "table") then
		self.TextFuncs = {};
	end
	self.TextFuncs[Name] = Func;
end

local TextSubs = FlexBar2.OO.Mixin 
{
	"LoadSubs", -- Load the text subs
	"RegisterTextSubs", -- The button specific part of RegisterTextSub
	"UpdateTextSubs", -- Update certain subs depending on what event triggered this
	"UpdateTextField", -- Update all text subs of one text field
	"SetTextSub", -- Set the text of a sub
};


function TextSubs:Activate()
	self.Frame.Text1 = self.Frame:CreateFontString(nil, "ARTWORK", "NumberFontNormal"):SetJustifyH("LEFT"):SetPoint("TOPLEFT", self.Frame, "TOPLEFT");
	self.Frame.Text2 = self.Frame:CreateFontString(nil, "ARTWORK", "NumberFontNormal"):SetJustifyH("RIGHT"):SetPoint("TOPRIGHT", self.Frame, "TOPRIGHT");
	self.Frame.Text3 = self.Frame:CreateFontString(nil, "ARTWORK", "NumberFontNormal"):SetJustifyH("RIGHT"):SetPoint("BOTTOMRIGHT", self.Frame, "BOTTOMRIGHT");
	self.Frame.Text4 = self.Frame:CreateFontString(nil, "ARTWORK", "NumberFontNormal"):SetJustifyH("LEFT"):SetPoint("BOTTOMLEFT", self.Frame, "BOTTOMLEFT");
	self.Frame.Text5 = self.Frame:CreateFontString(nil, "ARTWORK", "NumberFontNormal"):SetJustifyH("CENTER"):SetPoint("CENTER", self.Frame, "CENTER");
	self:RegisterTextSubs();
	self:LoadSubs();
	self:RegisterEvent("FB2_LoadSettings", "LoadSubs");
end

function TextSubs:LoadSubs()
	if(type(self.Settings.TextSubs) == "table") then
		for n,v in ipairs(self.Settings.TextSubs) do
			self:SetTextSub(n, v);
		end
	end
end

function TextSubs:RegisterTextSubs()
	for Triggers, Name in pairs(FlexBar2.TextSubs) do
		if(type(Triggers) == "string") then
			self:RegisterEvent(Triggers, "UpdateTextSubs");
		elseif(type(Triggers) == "table") then
			for n,v in ipairs(Triggers) do
				self:RegisterEvent(v, "UpdateTextSubs");
			end
		else
			return false;
		end
	end
end

function TextSubs:UpdateTextSubs(Event)
	if(type(self.TextSubs) ~= "table") then
		return;
	end
	for Name, Func in pairs(FlexBar2.TextSubs[Event]) do
		for Field = 1,5 do
			if(type(self.TextSubs[Field]) == "table") then
				for Key, Prefix in ipairs(self.TextSubs[Field].PrefixList) do
					if(Prefix == Name) then
						self.Frame["Text" .. Field]:SetText(Func(self, self.TextSubs[Field].SuffixList[Key]));
					end
				end
			end
		end
	end
end

function TextSubs:UpdateTextField(Field)
	for Key, Prefix in ipairs(self.TextSubs[Field].PrefixList) do
		if(type(FlexBar2.TextFuncs[Prefix]) == "function") then
			self.Frame["Text" .. Field]:SetText(FlexBar2.TextFuncs[Prefix](self, self.TextSubs[Field].SuffixList[Key]));
		end
	end
end

function TextSubs:SetTextSub(Field, Text, Save)
	if(Field > 5 or Field < 1 or type(Text) ~= "string") then
		return;
	end
	if(type(self.TextSubs) ~= "table") then
		self.TextSubs = {};
	end
	if(type(self.TextSubs[Field]) ~= "table") then
		self.TextSubs[Field] = {};
	end
	local PrefixList, SuffixList = {}, {};
	for Match in string.gmatch(Text, "$%((.-)%)") do
		local _, _, Prefix, Suffix = string.find(Match, "(%a*) ?(.*)");
		table.insert(PrefixList, Prefix);
		table.insert(SuffixList, Suffix);
	end
	self.TextSubs[Field].Text = Text;
	self.TextSubs[Field].PrefixList = PrefixList;
	self.TextSubs[Field].SuffixList = SuffixList;
	if(Save) then
		if(type(self.Settings.TextSubs) ~= "table") then
			self.Settings.TextSubs = {};
		end
		self.Settings.TextSubs[Field] = Text;
	end
	self:UpdateTextField(Field);
end

FlexBar2:RegisterButtonClass(TextSubs);


-- Default TextSub registering --

FlexBar2:RegisterTextSub("itemcount", "BAG_UPDATE", 
function(Button, Item)
	if(Item ~= "") then
		return GetItemCount(Item);
	else
		if(Button.Settings.Attributes.type == "action" and IsConsumableAction(Button.Settings.Attributes.action)) then
			return GetActionCount(Button.Settings.Attributes.action);
		end
	end
end);

FlexBar2:RegisterTextSub("cd", "FB2_UPDATE", 
function(Button)
	if(type(Button.Settings) == "table" and type(Button.Settings.Attributes) == "table" and Button.Settings.Attributes.type == "action") then
		local start, duration, enable = GetActionCooldown(Button.Settings.Attributes.action);
		if(duration > 1.5 and start ~= 0) then -- Ignore global cooldown or no cooldown
			local TimeLeft = (duration-(GetTime()-start));
			if(TimeLeft >= 60) then
				return(math.floor ((TimeLeft / 60) + 0.5) .. "m");
			else
				if(TimeLeft == 0) then
					return "";
				else
					return(math.floor (TimeLeft) .. "s");
				end
			end
		end
	end
	return "";
end);

FlexBar2:RegisterTextSub("name", nil,
function(Button)
	return Button.Button or "";
end);

FlexBar2:RegisterTextSub("key", "FB2_BINDINGS_UPDATE",
function(Button)
	if(type(Button.Settings.Bindings) == "table") then
		return Button.Settings.Bindings[1] or "";
	else
		return "";
	end
end);
